import { MiddlewareConsumer, Module, NestModule } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { GraphQLModule } from '@nestjs/graphql';
import { TypeOrmModule } from '@nestjs/typeorm';
import { AppController } from './app.controller';
import { AppResolver } from './app.resolver';
import { AppService } from './app.service';
import configuration from './commons/config/configuration';
import { ParseBodyMiddleware } from './commons/middleware/parse-body.middleware';
import { ArticlesModule } from './articles/articles.module';
import { EtcdModule } from '@nestjs-lib/etcd3';
import { CommonsModule } from './commons/commons.module';
import { TagsModule } from './tags/tags.module';

@Module({
  imports: [
    ConfigModule.forRoot({
      load: [configuration],
      isGlobal: true,
    }),
    TypeOrmModule.forRootAsync({
      useFactory: (configService: ConfigService) => ({
        type: 'postgres',
        host: configService.get<string>('db.postgres.host'),
        port: configService.get<number>('db.postgres.port'),
        username: configService.get<string>('db.postgres.username'),
        password: configService.get<string>('db.postgres.password'),
        database: configService.get<string>('db.postgres.database'),
        synchronize: false,
        autoLoadEntities: true,
      }),
      inject: [ConfigService],
    }),
    GraphQLModule.forRootAsync({
      useFactory: (configService: ConfigService) => ({
        debug: configService.get<string>('env') !== 'prod',
        playground: true,
        autoSchemaFile: true,
        installSubscriptionHandlers: true,
      }),
      inject: [ConfigService],
    }),
    EtcdModule.forRootAsync({
      useFactory: (configService: ConfigService) => ({
        hosts: configService.get<string>('db.etcd.hosts', 'localhost:2379'),
      }),
      inject: [ConfigService],
    }),
    CommonsModule,
    ArticlesModule,
    TagsModule,
  ],
  controllers: [AppController],
  providers: [AppService, AppResolver],
})
export class AppModule implements NestModule {
  public configure(consumer: MiddlewareConsumer): void {
    consumer.apply(ParseBodyMiddleware).forRoutes('*');
  }
}
